@@ -4,25 +4,25 @@ utility functions for asteval
    Matthew Newville <newville@cars.uchicago.edu>,
    The University of Chicago
 """
+from __future__ import division, print_function
 import ast
-import sys
+from sys import exc_info
 import re
 
-RESERVED_WORDS = ('and', 'as', 'assert', 'break', 'continue', 'def',
-                  'del', 'elif', 'else', 'except', 'finally', 'for',
-                  'from', 'if', 'import', 'in', 'is', 'not', 'or',
-                  'pass', 'print', 'raise', 'return', 'try', 'while',
-                  'group', 'end', 'endwhile', 'endif', 'endfor',
-                  'endtry', 'enddef', 'True', 'False', 'None')
+RESERVED_WORDS = ('and', 'as', 'assert', 'break', 'class', 'continue',
+                  'def', 'del', 'elif', 'else', 'except', 'exec',
+                  'finally', 'for', 'from', 'global', 'if', 'import', 'in',
+                  'is', 'lambda', 'not', 'or', 'pass', 'print', 'raise',
+                  'return', 'try', 'while', 'with', 'True', 'False',
+                  'None', 'eval', 'execfile', '__import__', '__package__')
 
-NAME_MATCH = re.compile(r"[a-z_][a-z0-9_]*$").match
+NAME_MATCH = re.compile(r"[a-zA-Z_][a-zA-Z0-9_]*$").match
 
 def valid_symbol_name(name):
     "input is a valid name"
-    lname = name[:].lower()
-    if lname in RESERVED_WORDS:
+    if name in RESERVED_WORDS:
         return False
-    return NAME_MATCH(lname) is not None
+    return NAME_MATCH(name) is not None
 
 # inherit these from python's __builtins__
 FROM_PY = ('ArithmeticError', 'AssertionError', 'AttributeError',
@@ -48,12 +48,12 @@ FROM_PY = ('ArithmeticError', 'AssertionError', 'AttributeError',
            'sum', 'tuple', 'type', 'zip')
 
 # inherit these from python's math
-FROM_MATH = ('acos', 'acosh', 'asin', 'asinh', 'atan', 'atan2',
-             'atanh', 'ceil', 'copysign', 'cos', 'cosh', 'degrees',
-             'e', 'exp', 'fabs',  'factorial', 'floor', 'fmod',
-             'frexp', 'fsum', 'hypot', 'isinf', 'isnan', 'ldexp',
-             'log', 'log10', 'log1p', 'modf', 'pi', 'pow', 'radians',
-             'sin',  'sinh', 'sqrt', 'tan', 'tanh', 'trunc')
+FROM_MATH = ('acos', 'acosh', 'asin', 'asinh', 'atan', 'atan2', 'atanh',
+             'ceil', 'copysign', 'cos', 'cosh', 'degrees', 'e', 'exp',
+             'fabs', 'factorial', 'floor', 'fmod', 'frexp', 'fsum',
+             'hypot', 'isinf', 'isnan', 'ldexp', 'log', 'log10', 'log1p',
+             'modf', 'pi', 'pow', 'radians', 'sin', 'sinh', 'sqrt', 'tan',
+             'tanh', 'trunc')
 
 # inherit these from numpy, if available
 FROM_NUMPY = ('Inf', 'NAN', 'abs', 'absolute', 'add', 'alen', 'all',
@@ -83,11 +83,11 @@ FROM_NUMPY = ('Inf', 'NAN', 'abs', 'absolute', 'add', 'alen', 'all',
               'diag_indices_from', 'diagflat', 'diagonal', 'diff',
               'digitize', 'disp', 'divide', 'dot', 'double', 'dsplit',
               'dstack', 'dtype', 'e', 'ediff1d', 'emath', 'empty',
-              'empty_like', 'equal', 'errstate', 'exp', 'exp2',
-              'expand_dims', 'expm1', 'extract', 'eye', 'fabs', 'fft',
-              'fill_diagonal', 'find_common_type', 'finfo', 'fix',
-              'flatiter', 'flatnonzero', 'flexible', 'fliplr', 'flipud',
-              'float', 'float32', 'float64', 'float_', 'floating', 'floor',
+              'empty_like', 'equal', 'exp', 'exp2', 'expand_dims', 'expm1',
+              'extract', 'eye', 'fabs', 'fft', 'fill_diagonal',
+              'find_common_type', 'finfo', 'fix', 'flatiter',
+              'flatnonzero', 'flexible', 'fliplr', 'flipud', 'float',
+              'float32', 'float64', 'float_', 'floating', 'floor',
               'floor_divide', 'fmax', 'fmin', 'fmod', 'format_parser',
               'frexp', 'frombuffer', 'fromfile', 'fromfunction',
               'fromiter', 'frompyfunc', 'fromregex', 'fromstring', 'fv',
@@ -152,7 +152,6 @@ NUMPY_RENAMES = {'ln':'log', 'asin':'arcsin', 'acos':'arccos',
                  'atan':'arctan', 'atan2':'arctan2', 'atanh':'arctanh',
                  'acosh':'arccosh', 'asinh':'arcsinh'}
 
-
 OPERATORS = {ast.Is:     lambda a, b: a is b,
              ast.IsNot:  lambda a, b: a is not b,
              ast.In:     lambda a, b: a in b,
@@ -186,41 +185,51 @@ def op2func(op):
     "return function for operator nodes"
     return OPERATORS[op.__class__]
 
-class ExceptionHolder:
+class ExceptionHolder(object):
     "basic exception handler"
-    def __init__(self, node, msg='', expr=None,
-                 py_exc=(None, None)):
+    def __init__(self, node, exc=None, msg='', expr=None, lineno=None):
         self.node   = node
         self.expr   = expr
         self.msg    = msg
-        self.py_exc = py_exc
-        self.exc_info = sys.exc_info()
+        self.exc    = exc
+        self.lineno = lineno
+        self.exc_info = exc_info()
+        if self.exc is None and self.exc_info[0] is not None:
+            self.exc = self.exc_info[0]
+        if self.msg is '' and self.exc_info[1] is not None:
+            self.msg = self.exc_info[1]
 
     def get_error(self):
         "retrieve error data"
-        node = self.node
-        node_col_offset = 0
-
-        if node is not None:
+        col_offset = -1
+        if self.node is not None:
             try:
-                node_col_offset = self.node.col_offset
-            except:
+                col_offset = self.node.col_offset
+            except AttributeError:
                 pass
+        try:
+            exc_name = self.exc.__name__
+        except AttributeError:
+            exc_name = str(self.exc)
+        if exc_name in (None, 'None'):
+            exc_name = 'UnknownError'
 
-        exc_text = str(self.exc_info[1])
-        if exc_text in (None, 'None'):
-            exc_text = ''
-        expr = self.expr
+        out = ["   %s" % self.expr]
+        if col_offset > 0:
+            out.append("    %s^^^" % ((col_offset)*' '))
+        out.append(str(self.msg))
+        return (exc_name, '\n'.join(out))
 
-        out = []
-        if len(exc_text) > 0:
-            out.append(exc_text)
-        else:
-            py_etype, py_eval = self.py_exc
-            if py_etype is not None and py_eval is not None:
-                out.append("%s: %s" % (py_etype, py_eval))
+class NameFinder(ast.NodeVisitor):
+    """find all symbol names used by a parsed node"""
+    def __init__(self):
+        self.names = []
+        ast.NodeVisitor.__init__(self)
 
-        out.append("    %s" % expr)
-        if node_col_offset > 0:
-            out.append("    %s^^^" % ((node_col_offset)*' '))
-        return (self.msg, '\n'.join(out))
+    def generic_visit(self, node):
+        nodename = node.__class__.__name__.lower()
+        if nodename == 'name':
+            if (node.ctx.__class__ == ast.Load and
+                node.id not in self.names):
+                self.names.append(node.id)
+        ast.NodeVisitor.generic_visit(self, node)
